@@ -58,6 +58,10 @@
 #include <dirsel.h>
 #ifdef LANGREC
 #include <decodage_usr.h>
+#include <dicoMySQL_def.h>
+#include <html_content.h>
+#include <html.h>
+#include <write_file.h>
 #endif /* LANGREC */
 #include <WebbaseDl.h>
 #include <WebbaseHook.h>
@@ -151,6 +155,10 @@ static struct option long_options[] =
   {"allow", 1, 0, 0},
   /*- <string[ string...]> Add strings as robots.txt Disallow clauses. */
   {"disallow", 1, 0, 0},
+  /*- <string[ string...]> Add strings as robots.txt Regex Allow clauses. */
+  {"regex_allow", 1, 0, 0},
+  /*- <string[ string...]> Add strings as robots.txt Regex Disallow clauses. */
+  {"regex_disallow", 1, 0, 0},
   /*- <agent> Agent HTTP field content (default webbase/5.11) */
   {"agent", 1, 0, 0},
   /*- Wait for every URLs from a server to be loaded before going to the next */
@@ -163,9 +171,51 @@ static struct option long_options[] =
   {"update_rebuild", 0, 0, 0},
   /*- <number> Only get <number> url at a time from the base (default 200) */
   {"candidate_limit", 1, 0, 0},
+  /*- <path> directory where the URLs copies are stored (default /var/lib/webbase) */
+  {"file_cache", 1, 0, 0},
   {0, 0, 0, CRAWL_OPTIONS}
 };
 
+static struct option_help long_options_help[] =
+{
+  {"babil_crawl", "very verbose crawl"},
+  {"verbose_crawl","medium verbose crawl" },
+  {"quiet_crawl", "silent crawl"},
+  {"verbose_dirsel", "verbose allow/disallow handling"},
+  {"no_hook", "do not call inverted index hooks"},
+  {"hook <name>", "use <name> inverted index library (default mifluz)"},
+  {"url_max_size <bytes>", "size in bytes of the longest url (default 255)"},
+  {"size_limit <bytes>", "maximum size in bytes of url buffer (default 100K)"},
+  {"min <bytes>", "minimun size of a document (default none)"},
+  {"depth <number>", "maximum number of pages retrieved from starting point (default 2)"},
+  {"level <number>", "maximum number of hops from the starting point (default none)"},
+  {"timeout <sec>", "I/O timeout (default 60)"},
+  {"loaded_delay <days>", "number of days to wait for loading again a page successfully loaded (default 7)"},
+  {"modified_delay <days>", "number of days to wait for loading again a page not modified since the last time it was checked (default 15)"},
+  {"not_found_delay <days>", "number of days to wait for loading again a page not found since the last time it was checked (default 30)"},
+  {"timeout_delay <days>", "number of days to wait for loading again a page that timeouted the last time it was checked (default 30)"},
+  {"robot_delay <sec>", "number of seconds to wait between two loads on the same server (default 60)"},
+  {"auth <user:password>", "auth part for all urls to load"},
+  {"accept <mime types>", "accepted mime types"},
+  {"filter <regexp>", "regular expression that every loaded pages must match"},
+  {"noheuristics", "disable all loading heuristics"},
+  {"unescape", "do not escape URLs sent to servers"},
+  {"norobot_exclusion", "do not comply with robot exclusion protocol"}, 
+  {"tags", "mark each URL indexed with tag"},
+  {"allow <list>", "allow a list of prefixes"},
+  {"disallow <list>", "disallow a list of prefixes"},
+  {"regex_allow <list>", "allow a list of regexs"},
+  {"regex_disallow <list>", "disallow a list of regexs"},
+  {"agent <agent>", "Agent HTTP field content (default webbase/5.12)"},
+  {"sleepy", "wait for every URLs from a server to be loaded before going to the next"},
+  {"sticky", "do not give up on server even if it timesout"},
+  {"update", "disable update heuristics"},
+  {"update_rebuild", "rebuild the graph for the server, even if update is not needed"},
+  {"candidate_limit <number>", "only get <number> url at a time from the base (default 200)"},
+  {"file_cache <path>", "directory where the URLs copies are stored (default /var/lib/webbase)"},
+  {"0", ""}
+};
+
 struct option* crawl_options(struct option options[])
 {
   getopt_merge(options, webbase_options(options));
@@ -176,6 +226,16 @@ struct option* crawl_options(struct option options[])
   return long_options;
 }
 
+struct option_help* crawl_help_options(struct option_help options_help[])
+{
+  getopt_help_merge(options_help, webbase_help_options(options_help));
+  getopt_help_merge(options_help, webtools_help_options(options_help));
+  getopt_help_merge(options_help, robots_help_options(options_help));
+  getopt_help_merge(options_help, cookies_help_options(options_help));
+
+  return long_options_help;
+}
+
 #define MAX_OPTIONS 100
 #define APPLICATION_OPTIONS		0x8000000
 
@@ -273,6 +333,9 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
 	} else if(!strcmp(options[option_index].name, "update_rebuild")) {
 	  params->update_rebuild = 1;
 	  break;
+	} else if(!strcmp(options[option_index].name, "file_cache")) {
+	  uri_set_root(optarg);
+	  break;
 	} else if(!strcmp(options[option_index].name, "robot_delay")) {
 	  start->robot_delay = atoi(optarg);
 	  break;
@@ -287,6 +350,8 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
         S(filter,filter_length,WEBBASE_URL_START_FILTER);
         S(allow,allow_length,WEBBASE_URL_START_ALLOW);
         S(disallow,disallow_length,WEBBASE_URL_START_DISALLOW);
+        S(regex_allow,regex_allow_length,WEBBASE_URL_START_REGEX_ALLOW);
+        S(regex_disallow,regex_disallow_length,WEBBASE_URL_START_REGEX_DISALLOW);
 #undef S
 	} else if(!strcmp(options[option_index].name, "no_hook")) {
 	  params->no_hook = 1;
@@ -298,6 +363,8 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
 	  if(!function)
 	    fprintf(stderr, "crawl::crawl_alloc: cannot load hook %s\n", optarg);
 	  else 
+	    if (verbose)
+	      fprintf(stderr,"crawl::crawl_alloc: initializing hook");
 	    params->hook = (WebbaseHook*)(*function)();
 	  break;
 	}
@@ -316,6 +383,10 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
     http_verbose(verbose);
   }
 
+  if(uri_get_root()[0] == '\0') {
+    uri_set_root(DEFAULT_FILE_CACHE);
+  }
+  
   /*
    * Create the TCP/IP access object
    */
@@ -368,7 +439,11 @@ static void crawl_rebuild_1(char* paramsp, webbase_url_t* webbase_url)
   webbase_lock_ignore(params->base, webbase_url->w_rowid, "url");
   if(params->hook->Ok(webbase_url)) {
     char query[256];
+#if 0
     char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
+#else
+    char* path = webbase_url_file(webbase_url);
+#endif
     if(file_exists(path)) {
       params->hook->Server(robots_url2server_id(params->base, webbase_url->w_url));
       params->hook->Insert(webbase_url);
@@ -396,17 +471,39 @@ static void crawl_rebuild_1(char* paramsp, webbase_url_t* webbase_url)
 }
 
 /*
- * Re-index everything
+ * Re-index everything or just of a part of the documents if the -where_url option
+ * is specified
  */
-void crawl_rebuild(crawl_params_t* params, int resume)
+void crawl_rebuild(crawl_params_t* params, int resume, char *where)
 {
   if(resume > 1) {
     fprintf(stderr, "resuming rebuild\n");
   } else {
-    params->hook->RebuildStart();
+    if (verbose > 1)
+      fprintf(stderr,"starting crawl_rebuild\n");
+
+    if (params->no_hook)
+      {
+	fprintf(stderr,"rebuild functionnality require hooks library");
+	exit(-1);
+      }
+    
+    params->hook->RebuildInit();
+  }
+    
+  params->hook->RebuildStart();
+
+  /*
+   * compose where clause
+   */
+  char* regexp_where = smalloc(128 + (where ? strlen(where) : 0));
+  if(where && where[0] != '\0') {
+    sprintf(regexp_where, "where %s and hookid = 0", where);
+  } else {
+    sprintf(regexp_where, "where hookid = 0");
   }
 
-  webbase_walk_url(params->base, "where hookid = 0", crawl_rebuild_1, (char*)params, WEBBASE_GET_URL_LIGHT);
+  webbase_walk_url(params->base, regexp_where, crawl_rebuild_1, (char*)params, WEBBASE_GET_URL_LIGHT);
 
   params->hook->RebuildEnd();
 }
@@ -834,6 +931,12 @@ static void hp_load_mirror(char* paramsp, webbase_url_start_t* start)
     dirsel_allow(start->url, start->allow, DIRSEL_LOAD);
   if(start->info & WEBBASE_URL_START_DISALLOW)
     dirsel_disallow(start->url, start->disallow, DIRSEL_LOAD);
+
+  if(start->info & WEBBASE_URL_START_REGEX_ALLOW)
+    dirsel_regex_allow(start->url, start->regex_allow, DIRSEL_LOAD);
+  if(start->info & WEBBASE_URL_START_REGEX_DISALLOW)
+    dirsel_regex_disallow(start->url, start->regex_disallow, DIRSEL_LOAD);
+                                         
   /*
    * Key to directory selection (robots & manual)
    */
@@ -1485,7 +1588,11 @@ static webbase_url_t* mirror_2(crawl_context_t* context, char* url)
    * Force crawling if the document is not in the cache.
    */
   if(context->touch && (webbase_url->w_info & WEBBASE_URL_INFO_CONTENT)) {
+#if 0
     char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
+#else
+    char* path = webbase_url_file(webbase_url);
+#endif
     if(path) {
       if(!file_exists(path)) {
 	if(verbose > 1) fprintf(stderr, "\tmirror: touching\n");
@@ -1776,7 +1883,11 @@ static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_
 
     {
 	int isdir;
+#if 0
 	char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
+#else
+	char* path = webbase_url_file(webbase_url);
+#endif
 	int code;
 
 	if(path) {
@@ -1824,6 +1935,28 @@ static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_
     return 0;
 }
 
+#ifdef LANGREC
+static int html_content_collect_begin(int info, html_content_parsed_t* parsed, void* data)
+{
+  int ret = 1;
+
+  switch(info) {
+
+  case HTML_CONTENT_META:
+  case HTML_CONTENT_TITLE:
+    break;
+
+  case HTML_CONTENT_TEXT:
+    {
+      append_file((char *)data,(char *)parsed->buffer0,parsed->buffer0_length);
+    }
+    break;
+  }
+
+  return ret;
+}
+#endif
+
 /*
  * Crawl a single URL using HTTP
  */
@@ -1882,24 +2015,56 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	   error == WEBTOOLS_READER_TRUNC) {
 	  if(verbose > 1) fprintf(stderr, "\tloading %s : body read\n", url);
 	  webbase_url_hrefs_set(webbase_url, webbase_url->path, context->start->size_limit);
+
 #ifdef LANGREC
 	  {
+	    // language recognition
+	    // added by benoit orihuela (benoit.orihuela@IDEALX.com), on 2000/07/20
+	    ///////////////////////
+	    if (verbose>1) fprintf(stderr,"\tbeginning language recognition\n");
+
 	    char* raw_path = webbase_url->path;
-	    char* path = smalloc(strlen(raw_path) + 2);
-	    sprintf(path, "%sR", raw_path);
-	    if(bodyparse(raw_path, path, webbase_url->w_url, 1000000, 0, BODY_PARSE_UNACCENT) < 0) {
-	      fprintf(stderr, "mirror_http: cannot convert to text %s\n", raw_path);
-	    } else {
-	      char language[DECODAGE_TAILLE_MAX];
-	      textlang_in_file(path, language);
-	      memcpy(webbase_url->w_language, language, WEBBASE_LANGUAGE_LENGTH);
-	      webbase_url->w_language[WEBBASE_LANGUAGE_LENGTH] = '\0';
-	      if(verbose > 1) fprintf(stderr, "\tlanguage = %s\n", webbase_url->w_language);
-	      webbase_url->w_language[0] = '\0';
-	      unlink(path);
+	    html_content_t walk;
+
+	    // create a temporary file to store the text of the url being parsed
+	    char f_lang[64];
+	    sprintf(f_lang,"/tmp/file.langrec");
+	    write_file(f_lang,"",0);
+
+	    // fill the html_content_t structure
+	    html_content_reset(&walk);
+	    walk.parser.info = HTML_SOURCE_FILENAME;
+	    // path to a cached file
+	    walk.parser.source = raw_path;
+	    walk.parser.source_length = strlen(raw_path);
+	    // callback called for each href found
+	    walk.content_callback = html_content_collect_begin;
+	    // path to the temporay file where content text is written
+	    walk.content_data = (void *)f_lang;
+	    // parse the data stored in cache file
+	    if (!html_content_parse(&walk)) {
+  	      fprintf(stderr, "mirror_http: cannot convert to text %s\n", raw_path);
+  	    } else {
+	    char language[DECODAGE_TAILLE_MAX];
+	    // find language in temp file (module : langrec / file : textlang_in_file)
+#ifdef DEFAULT_TRIE_PATH
+	    textlang_in_file(f_lang, language, DEFAULT_TRIE_PATH);
+#else
+	    textlang_in_file(f_lang, language, NULL);
+#endif
+	    if (verbose>1) fprintf(stderr,"\tlanguage found : %s\n",language);
+
+	    if (language[0] != '\0')
+	      strncpy(webbase_url->w_language, language, WEBBASE_LANGUAGE_LENGTH);
+	    else
+	      strncpy(webbase_url->w_language, "unknown", WEBBASE_LANGUAGE_LENGTH);
+	    webbase_url->w_language[WEBBASE_LANGUAGE_LENGTH] = '\0';
+	    // erase temporary file
+	    char cmd[64];
+	    sprintf(cmd,"rm -f %s",f_lang);
+	    system(cmd);
 	    }
-	    free(path);
-	  }
+  	  }
 #endif /* LANGREC */
 	  webbase_url_content_length_fix(webbase_url, webbase_url->path);
 	  webbase_url_mtime_fix(webbase_url);
@@ -1914,10 +2079,16 @@ static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri
 	}
       } else if(!(webbase_url->w_info & WEBBASE_URL_INFO_CONTENT) ||
 		webbase_url->w_content_length <= 0) {
+#if 0
 	char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
+#else
+	char* path = webbase_url_file(webbase_url);
+#endif
 	if(path) {
 	  if(file_exists(path))
-	    rm(path);
+	    {
+	      rm(path);
+	    }
 	} else {
 	  fprintf(stderr, "cannot uncache %s, unable to resolve path (ignored)\n", webbase_url->w_url);
 	}
