@@ -47,7 +47,7 @@
 #include <dirsel.h>
 #include <uri.h>
 #include <salloc.h>
-
+#include <regex.h>                                                                                                                    
 /*
  * Search for match at beginning of string (anchored)
  */
@@ -76,6 +76,10 @@ typedef struct dirsel {
   int pool_size;
 } dirsel_t;
 
+// regexs management
+enum   regex_types { ALLOW, DISALLOW };
+static regex_t** regexs[2] = { NULL, NULL };
+static int regex_length[2];                                                                                                           
 /*
  * Instance object. Holds mapping between server names
  * and struct dirsel entries listing the allowed/disallowed strings.
@@ -84,6 +88,8 @@ typedef struct dirsel {
 typedef struct context {
   hash_t* allow;
   hash_t* disallow;
+  hash_t* regex_allow;
+  hash_t* regex_disallow;
   hash_t* robots_allow;
   hash_t* robots_disallow;
   uri_t* url_object;
@@ -96,8 +102,10 @@ static context_t context;
 static void dirsel_end_1(hash_t* table);
 static void dirsel_insert(hash_t* table, char* url, char* dirs, int flag);
 static int dirsel_allowed_1(char* comparable, dirsel_t* allow, dirsel_t* disallow);
+static int dirsel_allowed_2(char* comparable, dirsel_t* regex_allow, dirsel_t* regex_disallow);
 static char* dirsel_comparable(char* netpath, uri_t* url_object);
 static int dirsel_match(dirsel_t* entry, char* comparable);
+static int dirsel_regex_match(dirsel_t* entry, char* comparable, int regex_type);
 static dirsel_t* dirsel_alloc();
 static char* dirsel_netpath(uri_t* url_object);
 static dirsel_t* entry_find(hash_t* table, char* key);
@@ -114,6 +122,7 @@ static void hnode_free(hnode_t *node, void *)
 
 /*
  * Create a table for robots.txt Allow/Disallow attributes + user provided Allow/Disallow
+ * + user provided Regex Allow/Regex Disallow
  */
 void dirsel_init()
 {
@@ -123,6 +132,12 @@ void dirsel_init()
     hash_set_allocator(context.allow, 0, hnode_free, 0);
     context.disallow = hash_create(size, 0, 0);
     hash_set_allocator(context.disallow, 0, hnode_free, 0);
+
+    context.regex_allow = hash_create(size, 0, 0);
+    hash_set_allocator(context.regex_allow, 0, hnode_free, 0);
+    context.regex_disallow = hash_create(size, 0, 0);
+    hash_set_allocator(context.regex_disallow, 0, hnode_free, 0);
+
     context.robots_allow = hash_create(size, 0, 0);
     hash_set_allocator(context.robots_allow, 0, hnode_free, 0);
     context.robots_disallow = hash_create(size, 0, 0);
@@ -137,8 +152,12 @@ void dirsel_end()
   dirsel_end_1(context.disallow);
   dirsel_end_1(context.robots_allow);
   dirsel_end_1(context.robots_disallow);
+  dirsel_end_1(context.regex_allow);
+  dirsel_end_1(context.regex_disallow);
   uri_free(context.url_object);
   context.url_object = 0;
+  dirsel_destroy_regex (ALLOW);
+  dirsel_destroy_regex (DISALLOW);
 }
 
 void dirsel_verbose(int level)
@@ -176,6 +195,36 @@ void dirsel_disallow(char* url, char* dirs, int flag)
   dirsel_insert(context.disallow, netpath, dirs, flag);
 }
 
+/*
+ * Add a list of strings <dirs> to the user specified Regex Allow clauses
+ * Added by Olivier Girondel (olivier.girondel@IDEALX.com, on 2000/08/01
+ */
+void dirsel_regex_allow(char* regex, char* dirs, int flag)
+{
+  char* netpath;
+  dirsel_destroy_regex (ALLOW);
+  uri_realloc(context.url_object, regex, strlen(regex));
+  netpath = dirsel_netpath(context.url_object);
+  if(verbose) fprintf(stderr, "\tdirsel_regex_allow: %s (%s)\n", netpath, dirs);
+  dirsel_insert(context.regex_allow, netpath, dirs, flag);
+  dirsel_build_regex (context.regex_allow, netpath, ALLOW);
+}
+
+/*
+ * Add a list of strings <dirs> to the user specified Regex Disallow clauses
+ * Added by Olivier Girondel (olivier.girondel@IDEALX.com, on 2000/08/01
+ */
+void dirsel_regex_disallow(char* regex, char* dirs, int flag)
+{
+  char* netpath;
+  dirsel_destroy_regex (DISALLOW);
+  uri_realloc(context.url_object, regex, strlen(regex));
+  netpath = dirsel_netpath(context.url_object);
+  if(verbose) fprintf(stderr, "\tdirsel_regex_disallow: %s (%s)\n", netpath, dirs);
+  dirsel_insert(context.regex_disallow, netpath, dirs, flag);
+  dirsel_build_regex (context.regex_disallow, netpath, DISALLOW);
+}
+
 /*
  * Add a list of strings <dirs> to the robots.txt Allow clauses
  */
@@ -262,6 +311,12 @@ int dirsel_allowed(char* netpath, char* url)
       if(verbose) fprintf(stderr, "\tdirsel_allowed: search manual\n");
 
       ret = dirsel_allowed_1(comparable, entry_find(context.allow, netpath), entry_find(context.disallow, netpath));
+
+      if(ret) {
+        if(verbose) fprintf(stderr, "\tdirsel_allowed: search regexs\n");
+
+        ret = dirsel_allowed_2(comparable, entry_find(context.regex_allow, netpath), entry_find(context.regex_disallow, netpath));
+        }
     }
     return ret;
   }
@@ -300,6 +355,40 @@ static int dirsel_allowed_1(char* comparable, dirsel_t* allow, dirsel_t* disallo
   }
 }
 
+/*
+ * Return true if comparable is allowed according to regex-allow/disallow specifications.
+ * First allow is considered, then disallow.
+ * Added by Olivier Girondel (olivier.girondel@IDEALX.com, on 2000/08/01
+ */
+static int dirsel_allowed_2(char* comparable, dirsel_t* regex_allow, dirsel_t* regex_disallow)
+{
+  /*
+   * In allow table, allowed
+   */
+  if(regex_allow) {
+    if(verbose) fprintf(stderr, "\tdirsel_allowed_2: allow table found\n");
+    if(dirsel_regex_match(regex_allow, comparable, ALLOW)) {
+      if(verbose) fprintf(stderr, "\tdirsel_allowed_2: match found in allow table\n");
+      return 1;
+    }
+  }
+
+  /*
+   * Not in allow table and not in disallow, allowed
+   */
+  if(!regex_disallow) {
+    if(verbose) fprintf(stderr, "\tdirsel_allowed_2: no disallow table, therefore allowed\n");
+    return 1;
+  }
+  if(verbose) fprintf(stderr, "\tdirsel_allowed_2: disallow table found for '%s'\n", comparable);
+
+  {
+    int result = dirsel_regex_match(regex_disallow, comparable, DISALLOW);
+    if(verbose) fprintf(stderr, "\tdirsel_allowed_2: match %sfound in disallow table\n", (result ? "" : "not "));
+    return !result;
+  }
+}
+
 /*
  * Return netpath URL part into a string fit to compare with strings stored
  * in dirsel_t structures.
@@ -310,7 +399,7 @@ static char* dirsel_comparable(char* netpath, uri_t* url_object)
   int netpath_length = strlen(netpath);
 
   /*
-   * If they do not start the same, the do not compare.
+   * If they do not start the same, then do not compare.
    */
   if(strncmp(fake_netpath, netpath, netpath_length))
     return 0;
@@ -366,6 +455,49 @@ static dirsel_t* dirsel_alloc()
   return entry;
 }
 
+/*
+ * Search a prefix contained in comparable.
+ * Added by Olivier Girondel (olivier.girondel@IDEALX.com, on 2000/08/01
+ */
+static int dirsel_regex_match(dirsel_t* entry, char* comparable, int regex_type)
+{
+#define ERRSIZE 256
+  static char error[ERRSIZE];
+
+  /*
+    Olivier: quels flags passer  regexec ? cf regexec(3)
+  */
+
+  int i;
+  for(i = 0; i < regex_length[regex_type]; i++) {
+    /*
+     * Systematicaly skip the leading /
+     * If, by accident, the prefix does not start with /, take it completely.
+     * If the prefix is empty, ignore it.
+     */
+    if (regexs[regex_type][i]) { /* make sure the regex was correctly compiled */
+      if(verbose) fprintf(stderr, "\tdirsel_regex_match: match regex %s with %s\n", entry->prefix[i], comparable);
+      int result = 0;
+
+      result = regexec (regexs[regex_type][i], comparable, 0, NULL, 0 /* flags = ? */);
+
+      if(!result) {
+        if(verbose) fprintf(stderr, "\tdirsel_regex_match: %s match found regex %s\n", comparable, entry->prefix[i]);
+        return 1;
+      }
+      else {
+        if (result == REG_ESPACE) {
+          regerror (result, regexs[regex_type][i], error, ERRSIZE);
+          if (verbose)
+            fprintf(stderr, "\tdirsel_regex_match: failed regexec: %s\n", error);
+        }
+      }
+    }
+  }
+  if(verbose) fprintf(stderr, "\tdirsel_regex_match: %s no match\n", comparable);
+  return 0;
+}
+
 /*
  * Return netpath component from <url>
  */
@@ -408,3 +540,78 @@ static dirsel_t* entry_find(hash_t* table, char* key)
   hnode_t* node = hash_lookup(table, key);
   return node ? (dirsel_t*)hnode_get(node) : 0;
 }
+
+
+/*
+ * Destroy regex
+ * Added by Olivier Girondel (olivier.girondel@IDEALX.com, on 2000/08/01
+ */
+void dirsel_destroy_regex (int regex_type) {
+  if (verbose)
+    fprintf (stderr, "\tfreeing %sallow regexs ...", (regex_type) ? "dis" : "");
+  if (regexs[regex_type]) {
+    int i;
+    for (i = 0; i < regex_length[regex_type]; i++)
+      if (regexs[regex_type][i]) {
+        regfree (regexs[regex_type][i]);
+        free (regexs[regex_type][i]);
+      }
+
+    free (regexs[regex_type]);
+
+    regex_length[regex_type] = 0;
+    regexs[regex_type] = NULL;
+  }
+  if (verbose)
+    fprintf (stderr, "\tdone.\n");
+}
+
+
+/*
+ * Build regex
+ * Added by Olivier Girondel (olivier.girondel@IDEALX.com, on 2000/08/01
+ */
+void dirsel_build_regex (hash_t* table, char* key, int regex_type) {
+#define ERRSIZE 256
+  static char error[ERRSIZE];
+
+  /*
+    Olivier: quels flags passer a regcomp ? cf regcomp(3)
+  */
+
+  int i;
+  int ret;
+
+  hnode_t* node = hash_lookup(table, key);
+  dirsel_t* entry = (dirsel_t*)hnode_get(node);
+
+  regex_length[regex_type] = entry->prefix_length;
+  regexs[regex_type] = (regex_t**)malloc (regex_length[regex_type] * sizeof(regex_t*));
+
+
+  if (verbose)
+    fprintf (stderr, "\tcompiling %sallow regexs :\n", (regex_type) ? "dis" : "");
+
+  for (i = 0; i < entry->prefix_length; i++) {
+    regexs[regex_type][i] = (regex_t*)malloc (sizeof (regex_t));
+
+    if (verbose)
+      fprintf (stderr, "\t\t regex: %s ...", entry->prefix[i]);
+
+    ret = regcomp(regexs[regex_type][i], entry->prefix[i], REG_NOSUB | REG_EXTENDED);
+    if (ret) {
+      if (verbose) {
+        regerror (ret, regexs[regex_type][i], error, ERRSIZE);
+        fprintf (stderr, "failed: %s.\n", error);
+      }
+
+      regfree (regexs[regex_type][i]);
+      free (regexs[regex_type][i]);
+      regexs[regex_type][i] = NULL;
+    }
+    else {
+      if (verbose)
+        fprintf (stderr, "ok.\n");
+    }
+  }
+}
