@@ -19,10 +19,6 @@
 #ifndef _dirsel_h
 #define _dirsel_h
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #define DIRSEL_LOAD	0x01
 #define DIRSEL_RELOAD	0x02
 
@@ -30,14 +26,14 @@ void dirsel_init();
 void dirsel_end();
 void dirsel_allow(char* url, char* dirs, int flag);
 void dirsel_disallow(char* url, char* dirs, int flag);
+void dirsel_regex_allow(char* regex, char* dirs, int flag);
+void dirsel_regex_disallow(char* regex, char* dirs, int flag);
 void dirsel_robots_allow(char* netloc, char* dirs, int flag);
 void dirsel_robots_disallow(char* netloc, char* dirs, int flag);
 int dirsel_allowed(char* netpath, char* url);
 char* dirsel_key(char* url);
 void dirsel_verbose(int level);
-
-#ifdef __cplusplus
-}
-#endif
+void dirsel_destroy_regex (int regex_type);
+void dirsel_build_regex (hash_t* table, char* key, int regex_type);
 
 #endif /* _dirsel_h */
