@@ -21,15 +21,12 @@
 
 #include <webbase_url.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+typedef struct html_content_data {
+  webbase_url_t* webbase_url;
+  int extract_length;
+} html_content_data_t;
 
 int html_href(char* path, webbase_url_t* webbase_url, int size_limit);
 int html_content_begin(char* buffer, int buffer_length, webbase_url_t* webbase_url);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif /* _html_h */
