@@ -39,10 +39,10 @@ extern "C" {
 #define HTML_CONTENT_META		(HTML_CONTENT_METANAME|HTML_CONTENT_METACONTENT)
 
 typedef struct html_content_parsed {
-  char* buffer0;
+  unsigned char* buffer0;
   int buffer0_length;
   int buffer0_size;
-  char* buffer1;
+  unsigned char* buffer1;
   int buffer1_length;
   int buffer1_size;
 } html_content_parsed_t;
