@@ -58,11 +58,23 @@ static struct option long_options[] =
   {0, 0, 0, ROBOTS_OPTIONS}
 };
 
+static struct option_help long_options_help[] =
+{
+  /* These options set a flag. */
+  {"verbose_robots", "robots.txt loading and parsing related messages."},
+  {"0", ""}
+};
+
 struct option* robots_options(struct option[])
 {
   return long_options;
 }
 
+struct option_help* robots_help_options(struct option_help [])
+{
+  return long_options_help;
+}
+
 robots_t* robots_alloc(int argc, char** argv, struct option options[])
 {
   robots_t* params = params_alloc();
@@ -250,8 +262,14 @@ static int robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url_
 
     if(webbase_url &&
        (webbase_url->w_info & WEBBASE_URL_INFO_CONTENT) &&
-       webbase_url->w_content_length > 0)
-      robots_parse(uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH), &allow, &disallow);
+       webbase_url->w_content_length > 0) {
+#if 0
+      char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
+#else
+      char* path = webbase_url_file(webbase_url);
+#endif
+      robots_parse(path, &allow, &disallow);
+    }
 
     static_alloc(&query, &query_size, 128 + netloc_length + strlen(allow) + strlen(disallow));
 
