@@ -66,11 +66,31 @@ static struct option long_options[] =
   {0, 0, 0, WEBBASE_OPTIONS}
 };
 
+static struct option_help long_options_help[] =
+{
+  /* These options set a flag. */
+  {"verbose_webbase", "database manipulation related messages."},
+  {"base <base name>", "name of the mysql database to use for the Meta data information."},
+  {"user <name>", "name of the user to connect to database."},
+  {"password <password>", "password of the -user to connect to database."},
+  {"port <port>", "TCP/IP port to connect to database, if not default."},
+  {"host <hostname>", "hostname of the MySQL database."},
+  {"socket <file>", "Unix socket file full path name for local database connection."},
+  {"datadir", "absolute path name of the directory containing the database."},
+  {"net_buffer <size>", "size of the network buffer for communications with the database (default is 1Mb)."},
+  {"0", ""}
+};
+
 struct option* webbase_options(struct option [])
 {
   return long_options;
 }
 
+struct option_help* webbase_help_options(struct option_help [])
+{
+  return long_options_help;
+}
+
 webbase_t* webbase_alloc(int argc, char** argv, struct option options[])
 {
   webbase_t* base = webbase_alloc_1();
@@ -178,10 +198,10 @@ webbase_t* webbase_alloc(int argc, char** argv, struct option options[])
     {
       char* home = getenv("HOME");
       if(home) {
-	char* cnf_file = (char*)smalloc(strlen(home) + 32);
-	sprintf(cnf_file, "%s/.my.cnf", home);
-	mysql_options(&base->mysql, MYSQL_READ_DEFAULT_FILE, cnf_file);
-	free(cnf_file);
+	//char* cnf_file = (char*)smalloc(strlen(home) + 32);
+	//sprintf(cnf_file, "%s/.my.cnf", home);
+	//mysql_options(&base->mysql, MYSQL_READ_DEFAULT_FILE, cnf_file);
+	//free(cnf_file);
       }
     }
     
@@ -512,7 +532,7 @@ static webbase_url_start_t* webbase_get_start_1(webbase_t* base, char* quote, ch
   static char* query = 0;
   static int query_size = 0;
 
-  static_alloc(&query, &query_size, 256 + value_length);
+  static_alloc(&query, &query_size, 512 + value_length);
 
   sprintf(query, "select " WEBBASE_START_FIELDS " from start where %s = %s%.*s%s and disabled = 'no'", field, quote, value_length, value, quote);
   smysql_query(&base->mysql, query);
@@ -554,6 +574,10 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
   static int allow_size = 0;
   static char* disallow = 0;
   static int disallow_size = 0;
+  static char* regex_allow = 0;
+  static int regex_allow_size = 0;
+  static char* regex_disallow = 0;
+  static int regex_disallow_size = 0;
   static char* hook_info = 0;
   static int hook_info_size = 0;
   static char* url = 0;
@@ -565,6 +589,8 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
   int has_filter = start->info & WEBBASE_URL_START_FILTER;
   int has_allow = start->info & WEBBASE_URL_START_ALLOW;
   int has_disallow = start->info & WEBBASE_URL_START_DISALLOW;
+  int has_regex_allow = start->info & WEBBASE_URL_START_REGEX_ALLOW;
+  int has_regex_disallow = start->info & WEBBASE_URL_START_REGEX_DISALLOW;
   int has_hook_info = start->info & WEBBASE_URL_START_HOOK_INFO;
 
   webbase_url_start_t* default_start = &base->default_start;
@@ -582,6 +608,10 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
     sql_quote_char(&allow, &allow_size, start->allow, strlen(start->allow));
   if(has_disallow)
     sql_quote_char(&disallow, &disallow_size, start->disallow, strlen(start->disallow));
+  if(has_regex_allow)
+    sql_quote_char(&regex_allow, &regex_allow_size, start->regex_allow, strlen(start->regex_allow));
+  if(has_regex_disallow)
+    sql_quote_char(&regex_disallow, &regex_disallow_size, start->regex_disallow, strlen(start->regex_disallow));
   if(has_hook_info)
     sql_quote_char(&hook_info, &hook_info_size, start->hook_info, strlen(start->hook_info));
 
@@ -592,6 +622,8 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
 	       filter_size +
 	       allow_size +
 	       disallow_size +
+	       regex_allow_size +
+	       regex_disallow_size +
 	       hook_info_size);
 
   if(start->rowid != 0) {
@@ -615,13 +647,15 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
     S(robot_delay);
     S(delay);
 #undef S
-    sprintf(query + strlen(query), ", auth = %c%s%c, accept = %c%s%c, filter = %c%s%c, allow = %c%s%c, disallow = %c%s%c, hook_info = %c%s%c, count = %d",
+    sprintf(query + strlen(query), ", auth = %c%s%c, accept = %c%s%c, filter = %c%s%c, allow = %c%s%c, disallow = %c%s%c, regex_allow = %c%s%c, regex_disallow = %c%s%c, hook_info = %c%s%c, count = %d",
 #define S(f,hasf) (hasf ? '\'' : ' '), (hasf ? f : "null"), (hasf ? '\'' : ' ')
 	  S(auth,has_auth),
 	  S(accept,has_accept),
 	  S(filter,has_filter),
 	  S(allow,has_allow),
 	  S(disallow,has_disallow),
+	  S(regex_allow,has_regex_allow),
+	  S(regex_disallow,has_regex_disallow),
 	  S(hook_info,has_hook_info),
 	  start->count
 	  );
@@ -650,13 +684,15 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
     S(robot_delay);
     S(delay);
 #undef S
-    sprintf(query + strlen(query), "%c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %d",
+    sprintf(query + strlen(query), "%c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %c%s%c, %d",
 #define S(f,hasf) (hasf ? '\'' : ' '), (hasf ? f : "null"), (hasf ? '\'' : ' ')
 	  S(auth,has_auth),
 	  S(accept,has_accept),
 	  S(filter,has_filter),
 	  S(allow,has_allow),
 	  S(disallow,has_disallow),
+	  S(regex_allow,has_regex_allow),
+	  S(regex_disallow,has_regex_disallow),
 	  S(hook_info,has_hook_info),
 	  start->count
 	  );
@@ -697,6 +733,8 @@ void webbase_merge_start(webbase_url_start_t* to, webbase_url_start_t* from)
   S(filter,filter_length,WEBBASE_URL_START_FILTER);
   S(allow,allow_length,WEBBASE_URL_START_ALLOW);
   S(disallow,disallow_length,WEBBASE_URL_START_DISALLOW);
+  S(regex_allow,regex_allow_length,WEBBASE_URL_START_REGEX_ALLOW);
+  S(regex_disallow,regex_disallow_length,WEBBASE_URL_START_REGEX_DISALLOW);
   S(hook_info,hook_info_length,WEBBASE_URL_START_HOOK_INFO);
 #undef S
 }
@@ -708,6 +746,12 @@ void webbase_start_free(webbase_url_start_t* start)
   if(start->filter) free(start->filter);
   if(start->allow) free(start->allow);
   if(start->disallow) free(start->disallow);
+  /*
+   * added deallocation of regex_allow and regex_disallow (purify)
+   * by benoit orihuela (benoit.orihuela@IDEALX.com), 2000/08/04
+   */
+  if(start->regex_allow) free(start->regex_allow);
+  if(start->regex_disallow) free(start->regex_disallow); 
   if(start->hook_info) free(start->hook_info);
 }
 
@@ -732,6 +776,8 @@ void webbase_start_reset(webbase_url_start_t* start)
   if(start->filter) start->filter[0] = '\0';
   if(start->allow) start->allow[0] = '\0';
   if(start->disallow) start->disallow[0] = '\0';
+  if(start->regex_allow) start->regex_allow[0] = '\0';
+  if(start->regex_disallow) start->regex_disallow[0] = '\0';
   if(start->hook_info) start->hook_info[0] = '\0';
 
   start->count = 0;
@@ -797,6 +843,8 @@ static webbase_url_start_t* webbase_decode_start(webbase_t* base, MYSQL_RES *res
     S(filter, filter_length, WEBBASE_URL_START_FILTER);
     S(allow, allow_length, WEBBASE_URL_START_ALLOW);
     S(disallow, disallow_length, WEBBASE_URL_START_DISALLOW);
+    S(regex_allow, regex_allow_length, WEBBASE_URL_START_REGEX_ALLOW);
+    S(regex_disallow, regex_disallow_length, WEBBASE_URL_START_REGEX_DISALLOW);
     S(hook_info, hook_info_length, WEBBASE_URL_START_HOOK_INFO);
 #undef S
     } else if(!strcmp(field->name, "count")) {
@@ -967,7 +1015,7 @@ webbase_url_t* webbase_get_url_1(webbase_t* base, char* quote, char* field, unsi
   
   webbase_url_reset(webbase_url, WEBBASE_URL_RESET_TOTAL);
 
-  static_alloc(&query, &query_size, 256 + value_length);
+  static_alloc(&query, &query_size, 512 + value_length);
 
   sprintf(query, "select "WEBBASE_URL_FIELDS" from url where %s = %s%.*s%s", field, quote, value_length, value, quote);
   smysql_query(&base->mysql, query);
