@@ -58,6 +58,8 @@ create table start ( \n\
   filter varchar(255) binary, \n\
   allow varchar(255) binary, \n\
   disallow varchar(255) binary, \n\
+  regex_allow varchar(255) binary, \n\
+  regex_disallow varchar(255) binary, \n\
   hook_info varchar(255) binary, \n\
   sources set ('all'), \n\
   disabled enum ('yes', 'no') default 'no', \n\
@@ -625,7 +627,8 @@ create table locks ( \n\
 "insert into mime2ext values ('www/unknown', 'ppz,mv,ex1,ex2,ex3,ex4,ex5,ex6,') \n",
 "insert into mime2ext values ('www/unknown1', 'ex7,ex8,ex9,mp3,dat,tmp,wave,') \n",
 "insert into mime2ext values ('www/unknown2', 'sea,tbk,tgz,wks,cab,jbf,qtvr,') \n",
-"insert into mime2ext values ('www/unknown3', 'dbf,cop,moov,bz2,wdz') \n",
+"insert into mime2ext values ('www/unknown3', 'dbf,cop,moov,bz2,wdz,tap,') \n",
+"insert into mime2ext values ('www/unknown4', 'rar,wst,rm,max,') \n",
 "insert into mime2ext values ('x-world/x-vrml', 'wrl,vrz,flr,') \n"
 };
 
