@@ -21,15 +21,7 @@
 
 #include <mysql.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 void webbase_create(MYSQL* mysql, const char** schema, int schema_length);
 void webbase_schema();
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif /* webbase_create_h */
