@@ -183,23 +183,25 @@ typedef struct webbase_url_common {
 /*
  * Info field values
  */
-#define WEBBASE_URL_START_SLEEPY	0x00001
-#define WEBBASE_URL_START_UNESCAPE	0x00002
-#define WEBBASE_URL_START_AUTH		0x00004
-#define WEBBASE_URL_START_NOCOOKIE	0x00008
-#define WEBBASE_URL_START_ACCEPT	0x00010
-#define WEBBASE_URL_START_HEURISTICS	0x00020
-#define WEBBASE_URL_START_STICKY	0x00040
-#define WEBBASE_URL_START_FILTER	0x00080
-#define WEBBASE_URL_START_HOMEFREE	0x00100
-#define WEBBASE_URL_START_VIRGIN	0x00200
-#define WEBBASE_URL_START_EXPLORING	0x00400
-#define WEBBASE_URL_START_EXPLORED	0x00800
-#define WEBBASE_URL_START_UPDATING	0x01000
-#define WEBBASE_URL_START_IN_CORE	0x02000
-#define WEBBASE_URL_START_ALLOW		0x04000
-#define WEBBASE_URL_START_DISALLOW	0x08000
-#define WEBBASE_URL_START_HOOK_INFO	0x10000
+#define WEBBASE_URL_START_SLEEPY	 0x00001
+#define WEBBASE_URL_START_UNESCAPE	 0x00002
+#define WEBBASE_URL_START_AUTH		 0x00004
+#define WEBBASE_URL_START_NOCOOKIE	 0x00008
+#define WEBBASE_URL_START_ACCEPT	 0x00010
+#define WEBBASE_URL_START_HEURISTICS	 0x00020
+#define WEBBASE_URL_START_STICKY	 0x00040
+#define WEBBASE_URL_START_FILTER	 0x00080
+#define WEBBASE_URL_START_HOMEFREE	 0x00100
+#define WEBBASE_URL_START_VIRGIN	 0x00200
+#define WEBBASE_URL_START_EXPLORING	 0x00400
+#define WEBBASE_URL_START_EXPLORED	 0x00800
+#define WEBBASE_URL_START_UPDATING	 0x01000
+#define WEBBASE_URL_START_IN_CORE	 0x02000
+#define WEBBASE_URL_START_ALLOW		 0x04000
+#define WEBBASE_URL_START_DISALLOW	 0x08000
+#define WEBBASE_URL_START_HOOK_INFO	 0x10000
+#define WEBBASE_URL_START_REGEX_ALLOW	 0x20000
+#define WEBBASE_URL_START_REGEX_DISALLOW 0x40000
 
 #define WEBBASE_URL_START_STATE_MASK    0x1e00
 
@@ -259,6 +261,12 @@ typedef struct webbase_url_start {
   /* Additional robots.txt Disallow fields  */
   char* disallow;
   int disallow_length;
+  /* Additional Allow regex fields  */
+  char* regex_allow;
+  int regex_allow_length;
+  /* Additional Disallow regex fields  */
+  char* regex_disallow;
+  int regex_disallow_length;
   /* User defined string for full text indexing */
   char* hook_info;
   int hook_info_length;
@@ -408,5 +416,9 @@ void webbase_url_print(webbase_url_t* webbase_url);
  * Set modification time to now if not set
  */
 void webbase_url_mtime_fix(webbase_url_t* webbase_url);
+/*
+ * Return file name for storage
+ */
+char* webbase_url_file(webbase_url_t* webbase_url);
 
 #endif /* _webbase_url_h */
