@@ -68,11 +68,14 @@ typedef struct webtools_params {
   int timeout;
   /* Maximum total size of bytes read from input */
   int size_limit;
+  /* hostname -> IP cache. */
+  hash_t* host2ip;
   webtools_file_t files[WEBTOOLS_MAX_FD];
 } webtools_params_t;
 
 webtools_params_t* webtools_alloc(int argc, char** argv, struct option options[]);
 struct option* webtools_options(struct option options[]);
+struct option_help* webtools_help_options(struct option_help options_help[]);
 void webtools_free(webtools_params_t* params);
 /*
  * Open connection to <host>:<port> 
