@@ -35,7 +35,7 @@ class WebbaseGetopt {
   // Generic loop parsing options specific to this package.
   // Call handle_option for each option found.
   //
-  void Parse(int argc, char** argv);
+  virtual void Parse(int argc, char** argv);
   //
   // Handle an option. Returns 1 if the option was recognized
   // or 0 if it was not. If the option was recognized, it is 
