@@ -19,6 +19,14 @@
 #ifndef _file_size_h
 #define _file_size_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int file_size(char* filename);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* file_size_h */
